#include <errno.h>
#include "pthread.h"
#include "impl.h"

int pthread_mutex_init(pthread_mutex_t *mutex, const pthread_mutexattr_t *mattr)
{
	if(!(mutex->mut = CreateMutex(0, 0, 0))) {
		errno = EAGAIN;
		return -1;
	}
	return 0;
}

int pthread_mutex_destroy(pthread_mutex_t *mutex)
{
	if(!mutex->mut) {
		errno = EINVAL;
		return -1;
	}
	if(pthread_mutex_trylock(mutex) == -1) {
		errno = EBUSY;
		return -1;
	}
	while(pthread_mutex_unlock(mutex) != -1);

	CloseHandle(mutex->mut);
	return 0;
}

int pthread_mutex_trylock(pthread_mutex_t *mutex)
{
	if(!mutex->mut) {
		if(pthread_mutex_init(mutex, 0) == -1) {
			errno = EINVAL;
			return -1;
		}
	}

	if(WaitForSingleObject(mutex->mut, 0) == WAIT_TIMEOUT) {
		errno = EBUSY;
		return -1;
	}
	return 0;
}

int pthread_mutex_lock(pthread_mutex_t *mutex)
{
	if(!mutex->mut) {
		if(pthread_mutex_init(mutex, 0) == -1) {
			errno = EINVAL;
			return -1;
		}
	}

	WaitForSingleObject(mutex->mut, INFINITE);
	return 0;
}

int pthread_mutex_unlock(pthread_mutex_t *mutex)
{
	if(!mutex->mut) {
		if(pthread_mutex_init(mutex, 0) == -1) {
			errno = EINVAL;
			return -1;
		}
	}

	if(!ReleaseMutex(mutex->mut)) {
		errno = EPERM;
		return -1;
	}
	return 0;
}

int pthread_mutexattr_init(pthread_mutexattr_t *attr)
{
	return 0;	/* TODO */
}

int pthread_mutexattr_destroy(pthread_mutexattr_t *attr)
{
	return 0;	/* TODO */
}

int pthread_mutexattr_getpshared(const pthread_mutexattr_t *attr, int *pshared)
{
	return 0;	/* TODO */
}

int pthread_mutexattr_setpshared(pthread_mutexattr_t *attr, int pshared)
{
	return 0;	/* TODO */
}

